package com.objis.demohibernate.service;


import org.hibernate.Transaction;
import org.hibernate.classic.Session;
import org.hibernate.stat.Statistics;


import com.objis.demohibernate.domaine.Formation;
import com.objis.demohibernate.simple.util.HibernateUtil;


public class GestionnaireFormation {

	// Rendre persistant une formation 
	public Long creerFormation(Formation formation){
		
		Statistics stats = HibernateUtil.getSessionFactory().getStatistics(); // statistiques

		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = session.beginTransaction();

		Long formationId = null;
		try {
			session.saveOrUpdate(formation);
			tx.commit();
			formationId = formation.getId();
		} 
		catch (Exception e) {
			tx.rollback();
		} 
		finally{
			session.close();
		}
		
		stats.logSummary();                // statistiques

		System.out.println( "       getEntityNames = " + stats.getEntityNames()[0]);
		System.out.println( "    EntityDeleteCount = " + stats.getEntityDeleteCount());
		System.out.println( "  getEntityFetchCount = " + stats.getEntityFetchCount());
		System.out.println( " getEntityInsertCount = " + stats.getEntityInsertCount());
		System.out.println( "   getEntityLoadCount = " + stats.getEntityLoadCount());
		System.out.println( " getEntityUpdateCount = " + stats.getEntityUpdateCount());
		
		return formationId;
	}

	public void demoMerge1() {
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = session.beginTransaction();

		// 3 : Rcup Formation (Chargement d'une formation  partir de son Id=3)
		Formation formation = (Formation) session.get(Formation.class,new Long(3));
		
		tx.commit();
		session.close();
		
		// Ici Objet formation dtach !!!!
		formation.setTheme("ajax");
		
		Session session2 = HibernateUtil.getSessionFactory().openSession();
		Transaction tx2 = session2.beginTransaction();

		// 3 : Rcup Formation (Chargement d'une formation  partir de son Id=3)
		Formation formation2 = (Formation) session2.get(Formation.class,new Long(3));
		
		formation2.setTheme("SOA");
		session2.update(formation); // Gnre exception org.hibernate.NonUniqueObjectException: 

		tx2.commit();
		session2.close();
	}
	
	public void demoMerge2(){
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = session.beginTransaction();

		// 3 : Rcup Formation (Chargement d'une formation  partir de son Id=3)
		Formation formation = (Formation) session.get(Formation.class,new Long(3));
		
		tx.commit();
		session.close();
		
		// Ici Objet formation dtach !!!!
		formation.setTheme("ajax");
		
		Session session2 = HibernateUtil.getSessionFactory().openSession();
		Transaction tx2 = session2.beginTransaction();

		// 3 : Rcup Formation (Chargement d'une formation  partir de son Id=3)
		Formation formation2 = (Formation) session2.get(Formation.class,new Long(3));
		
		session2.merge(formation); // Pas d'exception  copie de formation dans formation2. RQ : formation2 non modifi. Rsultat en base : champ SPRING de enregistrement 3 formation remplac par ajax

		System.out.println("#######################################" );
		System.out.println("formation2 = Theme : " + formation2.getTheme() + " , dure : "+ formation2.getDuree() );
		System.out.println("#######################################" );
				
		tx2.commit();
		session2.close();
	}

	public void demoMerge3(){
		
		Session session = HibernateUtil.getSessionFactory().openSession();
		Transaction tx = session.beginTransaction();

		// 3 : Rcup Formation (Chargement d'une formation  partir de son Id=3)
		Formation formation = (Formation) session.get(Formation.class,new Long(3));
		
		tx.commit();
		session.close();
		
		// Ici Objet formation dtach !!!!
		formation.setTheme("ajax");
		
		Session session2 = HibernateUtil.getSessionFactory().openSession();
		Transaction tx2 = session2.beginTransaction();

		// 3 : Rcup Formation (Chargement d'une formation  partir de son Id=3)
		Formation formation2 = (Formation) session2.get(Formation.class,new Long(3));
		
		formation2.setTheme("SOA");
		formation2.setDuree(5);

		Formation formation3 = (Formation)session2.merge(formation); // Pas d'exception  

		System.out.println("#######################################" );
		System.out.println("formation2 = Theme : " + formation2.getTheme() + " , dure : "+ formation2.getDuree() );
		System.out.println("#######################################" );
		System.out.println("formation3 = Theme : " + formation3.getTheme() + " , dure : "+ formation3.getDuree() );
		System.out.println("#######################################" );
		
		tx2.commit();
		session2.close();
	}
	
	public Formation getFormation(Long id){		
		Session session = HibernateUtil.getSessionFactory().openSession();
		// Rcup Formation (Chargement d'une formation  partir de son Id)
		Formation formation = (Formation) session.get(Formation.class,id);		
		session.close();		
		return formation;
	}
}
